/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.usecases.remoting;

import java.io.IOException;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.model.SelectItem;
import org.apache.shale.remoting.faces.ResponseFactory;
import org.apache.shale.usecases.view.Domains;
import org.apache.shale.view.AbstractFacesBean;

/**
 * <p>Remotely executable business logic methods that complete the entire
 * response.</p>
 *
 * $Id: Business.java 476661 2006-11-18 23:44:11Z craigmcc $
 */
public class Business extends AbstractFacesBean {

    // ---------------------------------------------------------- Public Methods

   /**
    * <p>Generate an XML response from an array of JSF SelectItems. Those
    *    items represent a city/state pair that matches a zip code. The
    *    zip code is stored in the <i>zip</i> request parameter. That
    *    parameter was put in request scope by an Ajax call--see zipCode.jsp
    *    for more details. When the XML response is complete, the 
    *    <code>protected</code> method <code>selectItems</code> invokes
    *    <code>responseComplete()</code> on the Faces context, which ends
    *    the response.</p>
    */
   public void cityAndStateForZip() throws IOException {

       FacesContext context = getFacesContext();
       Map requestParams = getRequestParameterMap();
       String zip = (String)requestParams.get("zip");
       Domains domains = (Domains) getBean("domains");

       if (zip != null)
         selectItems(context, domains.getCityAndStateItems(zip));
       else
         selectItems(context, domains.getBlankCityAndStateItems());

   }

    /**
     * <p>Return the set of reported categories.</p>
     */
    public void listCategories() throws IOException {

        FacesContext context = getFacesContext();
        selectItems(context, supportedCategories(context));

    }


    /**
     * <p>Return the set of reported locales.</p>
     */
    public void listLocales() throws IOException {

        FacesContext context = getFacesContext();
        selectItems(context, supportedLocales(context));

    }



    // ------------------------------------------------------- Protected Methods


    /**
     * <p>Render an XML document containing the specified selection items
     * as the response to this request.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param items Selection items to be rendered
     */
    protected void selectItems(FacesContext context, SelectItem items[]) throws IOException {

        ResponseWriter writer =
                (new ResponseFactory()).getResponseWriter(context, "text/xml; charset=UTF-8");

        // Generate the response content
        writer.startDocument();
        writer.startElement("items", null);
        writer.write("\n");
        for (int i = 0; i < items.length; i++) {
            writer.startElement("item", null);
            writer.write("\n");
            Object value = items[i].getValue();
            if (value != null) {
                writer.startElement("value", null);
                writer.writeText(value, null);
                writer.endElement("value");
                writer.write("\n");
            }
            String label = items[i].getLabel();
            if (label != null) {
                writer.startElement("label", null);
                writer.writeText(label, null);
                writer.endElement("label");
                writer.write("\n");
            }
            String description = items[i].getLabel();
            if (description != null) {
                writer.startElement("description", null);
                writer.writeText(description, null);
                writer.endElement("description");
                writer.write("\n");
            }
            writer.endElement("item");
            writer.write("\n");
        }
        writer.endElement("items");
        writer.write("\n");

        context.responseComplete();

    }


    /**
     * <p>Return the set of legal supported categories.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    protected SelectItem[] supportedCategories(FacesContext context) {

        Domains domains = (Domains) getBean("domains");
        return domains.getSupportedCategories(context.getViewRoot().getLocale());

    }


    /**
     * <p>Return the set of legal supported locales.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    protected SelectItem[] supportedLocales(FacesContext context) {

        Domains domains = (Domains) getBean("domains");
        return domains.getSupportedLocales(context.getViewRoot().getLocale());

    }


}
